/*
 * \file   TI_MSPBoot_CI_PHYDL_USCI_I2C.c
 *
 * \brief  Driver for I2C Physical-DataLink layers using USCI
 *
 */
/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

//
// Include files
//
#include "msp430.h"
#include "TI_MSPBoot_Common.h"
#include "TI_MSPBoot_CI.h"
#include "TI_MSPBoot_AppMgr.h"
#include "TI_MSPBoot_CI_PHYDL.h"

#define I2C_CTL0_REG        UCB0CTL0
#define I2C_CTL1_REG        UCB0CTL1
#define I2C_OA_REG          UCB0I2COA
#define I2C_IFG_REG         UCB0IFG
#define I2C_TX_BUF          UCB0TXBUF
#define I2C_RX_BUF          UCB0RXBUF
#define I2C_STAT_REG        UCB0STAT

#define TIMER_CTL_REG       TA1CTL
#define TIMER_SRC_SEL       TASSEL_2  //SMCLK
#define TIMER_CLR           TACLR
#define TIMER_CCR_REG       TA1CCR0
#define TIMER_CCTL_REG      TA1CCTL0

//
//  Type Definitions
//
/*! State machine used by this communication interface 
 *   USCI doesn't require too much control, so we only check for an idle state
 *   and when receiving a packet 
 */
typedef enum {
    USCI_STATE_IDLE=0,          /*! Initialized state waiting for start */
    USCI_STATE_RECEIVING,       /*! Receiving packet */
}USCI_State_Machine_e;


//
//  Global variables
//
//    Note that these variables are assigned to a specific location in RAM
//    since they can also be used during application.
//    If not used by application, the location doesn't matter.
//
/*! Pointer to the Communication Interface callback structure 
 *   The NWK layer will define its callback structure and pass the pointer to
 *   this layer. An application can also declare its own structure and pass it
 *   to this layer in order to use the same interface.
 */
extern t_CI_Callback  * CI_Callback_ptr;

/*! State machine used by this interface. 
 */
extern USCI_State_Machine_e CI_State_Machine;

//
//  Local definitions
//
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
/*! Number of clock cycles for a millisecond. Clock runs from SMCLK/4=MCLK/4 */
#define MS_CYCLES                   ((MCLK/4)/1000)
/*! Number of clock cycles for a SMBus timeout (25 ms) */
#define TIMEOUT_CYCLES        (25*MS_CYCLES)
#endif

//
//  Local function prototypes
//
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
static void Timer_Timeout_Init(void);
static void Timer_Timeout_Start(void);
static void Timer_Timeout_Stop(void);
static uint8_t Timer_Timeout_Expired(void);
#endif

//
//  Function declarations
//
/******************************************************************************
*
 * @brief   Initializes the USCI I2C Slave interface
 *  - Sets corresponding GPIOs for I2C functionality
 *  - Resets and then configures the I2C
 *  - Initializes the I2C state machine
 *  - Initializes the I2C callbacks
 *  The NWK layer will define its callback structure and pass the pointer to
 *   this function. An application can also declare its own structure and call
 *   this function in order to use the same interface.
 *
 *  @param  CI_Callback     Pointer to Communication interface callbacks
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_Init(t_CI_Callback * CI_Callback)
{
    P3SEL |= BIT0 + BIT1;                       // Assign I2C pins to USCI_B0

    I2C_CTL1_REG |= UCSWRST;                    // Enable SW reset
    I2C_CTL0_REG = UCMODE_3 + UCSYNC;           // I2C Slave, synchronous mode
    I2C_OA_REG = CONFIG_CI_PHYDL_I2C_SLAVE_ADDR;     // set own (slave) address
    I2C_CTL1_REG &= ~UCSWRST;                   // Clear SW reset, resume operation
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
    Timer_Timeout_Init();
#endif
    // Initialize all callbacks
    CI_Callback_ptr = CI_Callback;
    // Init state machine
    CI_State_Machine = USCI_STATE_IDLE;
}

/******************************************************************************
 *
 * @brief   Disables the USCI module
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_disable(void)
{
    I2C_CTL1_REG |= UCSWRST;
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
    Timer_Timeout_Stop();
#endif
}

/******************************************************************************
 *
 * @brief   Enables the USCI module
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_reenable(void)
{
    I2C_CTL1_REG &= ~UCSWRST;
}

/******************************************************************************
*
 * @brief   Polls for USCI flags
 *  - Checks the Start, RX, TX, Stop flags and Timeout timer
 *  - This function calls the corresponding Callback functions and informs
 *    higher level layers, so they can handle data properly
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_Poll(void)
{
    uint8_t temp;
    uint8_t flag_ifg;
    
    // Read flags at the beginning of function 
    // New detected flags will be handled in the next poll
    flag_ifg = I2C_IFG_REG;

    if (flag_ifg & UCSTTIFG)   // Check for Start bit flag
    {
#ifdef CONFIG_CI_PHYDL_STOP_CALLBACK
        if ((CI_State_Machine != USCI_STATE_IDLE) &&
            (CI_Callback_ptr->StopCallback != NULL))
        {
            // Special case when a stop was cleared by a new Start
            I2C_IFG_REG &= ~UCSTPIFG;              // Clear stop flag
            #ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
                Timer_Timeout_Stop();           // Stop timeout timer
            #endif
            CI_Callback_ptr->StopCallback();  // Call Stop callback (if valid)
            CI_State_Machine = USCI_STATE_IDLE; // Reset state machine
            return;
        }
#endif
        // If no stop was expected/used, process the start flag
#ifdef CONFIG_CI_PHYDL_START_CALLBACK
        if (CI_Callback_ptr->StartCallback != NULL)
        {
            CI_Callback_ptr->StartCallback(); // Call Start callback (if valid)
        }
#endif
        CI_State_Machine = USCI_STATE_RECEIVING;    // Update state machine
        I2C_IFG_REG &= ~UCSTTIFG;                      // Clear start flag
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Start();                      // Refresh timer
#endif
    }
    else if (flag_ifg & UCRXIFG)     // Check the RX flag
    {
        temp = I2C_RX_BUF;                           // Get received byte
        if (CI_Callback_ptr->RxCallback != NULL)
        {
            // Call RX Callback (if valid) and send byte to upper layer
            CI_Callback_ptr->RxCallback(temp);      
        }
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Start();                      // Refresh timer
#endif
    }
    else if (flag_ifg & UCTXIFG)     // Check for TX flag
    {
        // Send ACK after byte reception
        if (CI_Callback_ptr->TxCallback != NULL)
        {
            // Call TXCallback (if valid) and get byte to send from upper layer
            CI_Callback_ptr->TxCallback(&temp);     
            I2C_TX_BUF = temp;                       // Send byte
        }
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Start();                      // Refresh timer
#endif
    }
    else if (flag_ifg & UCSTPIFG)      // Check for Stop bit flag
    {
        I2C_IFG_REG &= ~UCSTPIFG;
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Stop();                       // Stop timeout timer on end of frame
#endif
#ifdef CONFIG_CI_PHYDL_STOP_CALLBACK
        if (CI_Callback_ptr->StopCallback != NULL)
        {
            CI_Callback_ptr->StopCallback();      // Call End callback (if valid)
        }
#endif
        CI_State_Machine = USCI_STATE_IDLE;         // Reset the state machine
    }
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
    else if (Timer_Timeout_Expired() != 0x00)   // Check timeout timer
    {
        Timer_Timeout_Stop();
#ifdef CONFIG_CI_PHYDL_ERROR_CALLBACK
        if (CI_Callback_ptr->ErrorCallback != NULL)
        {
            // Call Error callback (if valid) and inform the upper layer
            CI_Callback_ptr->ErrorCallback(1);
        }
#endif
    }
#endif
}   // TI_MSPBoot_CI_PHYDL_Poll

#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
/******************************************************************************
*
 * @brief   Init timeout timer
 *  - Timer will run using SMCLK/4 in UP mode. Counts 25ms and is disabled by
 *    default
 *
 * @return  none
 *****************************************************************************/
static void Timer_Timeout_Init(void)
{
    // TA1 Uses SMCLK/4 in Up mode and is stopped by default
    TIMER_CTL_REG = TIMER_SRC_SEL + TIMER_CLR + MC_0 + ID_2;
    // Timer will count 25ms
    TIMER_CCR_REG = TIMEOUT_CYCLES;
    // TA1.0 used in compare mode without interrupts
    TIMER_CCTL_REG = 0;
}

/******************************************************************************
*
 * @brief   Starts timeout timer
 *
 * @return  none
 *****************************************************************************/
static void Timer_Timeout_Start(void)
{
    TIMER_CTL_REG = TIMER_SRC_SEL + TIMER_CLR + MC_1 + ID_2;    // Start timer in Up mode
    TIMER_CCTL_REG &= ~CCIFG;                         // Clear flag
}

/******************************************************************************
*
 * @brief   Stop timeout timer
 *
 * @return  none
 *****************************************************************************/
static void Timer_Timeout_Stop(void)
{
    TIMER_CTL_REG = TIMER_SRC_SEL + TIMER_CLR + MC_0;   // Stop timer
    TIMER_CCTL_REG &= ~CCIFG;                 // Clear flag
}

/******************************************************************************
*
 * @brief   Checks if timeout timer has expired
 *
 * @return  0 if not expired. !=0 if expired.
 *****************************************************************************/
static uint8_t Timer_Timeout_Expired(void)
{
    return (TIMER_CCTL_REG & CCIFG);          // Check timer flag
}

#endif  //CONFIG_CI_PHYDL_I2C_TIMEOUT

//  Constant table
//
/*! Peripheral Interface vectors for Application:
 *  These vectors are shared with application and can be used by the App to
 *   initialize and use the Peripheral interface
 */
#ifdef CONFIG_CI_PHYDL_COMM_SHARED
#pragma DATA_SECTION(Boot2App_Vector_Table, ".BOOT_APP_VECTORS")
#pragma RETAIN(Boot2App_Vector_Table)
const uint16_t Boot2App_Vector_Table[] =
{
    (uint16_t) &TI_MSPBoot_CI_PHYDL_Init,       /*! Initialization routine */
    (uint16_t) &TI_MSPBoot_CI_PHYDL_Poll,       /*! Poll routine */
};
#endif  //CONFIG_CI_PHYDL_COMM_SHARED
