# --COPYRIGHT--,BSD
#  Copyright (c) 2017, Texas Instruments Incorporated
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  *  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  *  Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
#  *  Neither the name of Texas Instruments Incorporated nor the names of
#     its contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --/COPYRIGHT--

#Perl script to generate to necessary files for an MSPBoot project
#   Check help for usage
#

use strict;
use warnings;
use POSIX qw/strftime/;
use Getopt::Long;
use List::Util qw[min max];
use File::Copy;
use File::Path qw(make_path remove_tree);

print( "\nMSPBootProjectCreator.pl V1.0 \n");
print( "Texas Instruments Inc 2017 \n");

# Variables needed for command line args
my $hdr_file;
my $lnk_file;
my $dual_image = 0;
my $I2C = 0;
my $UART = 0;
my $boot_size=0;
my $num_shared_vectors;
my $help = 0;

# Parse the command line arguments
GetOptions ("hdr=s"         => \$hdr_file,                 # string, required
			"lnk_file=s"    => \$lnk_file,                 # string, required
			"boot_size=s{1}" => \$boot_size,               # string  optional
            "shared_vectors=s{1}" => \$num_shared_vectors, # string, optional
			"I2C"           => \$I2C,                      # optional
			"UART"          => \$UART,                     # optional
			"dual_image"    => \$dual_image,               # optional
            "help" => \$help
            )      
  or die("Error in command line arguments\n");
  
# Check that all necesary data has been entered
if((!$hdr_file)||($help)||(!$lnk_file))
{
	print("Error in command line arguments\n");
	help();
    die;
}

#Get the device number from the provided default linker command file
my $device = getdeviceinFile($lnk_file);

if($device ne 0)
{
	$device = uc $device;
	print ("Device: ".$device."\n");
}
else
{
	die("Error: Invalid linker file - device name not found!\n");
}

#### Checks for correctness of command line arguments
{
	#Check if boot size is a mutliple of 1024 bytes
	if($boot_size)
	{
		if(hex($boot_size) % 0x400 != 0 || hex($boot_size) == 0)
		{
			die("Error: Boot size must be a multiple of 0x200!");
		}
	}

	# Check if a valid communication interface was specified
	if(!$I2C & !$UART)
	{
		die("Error: Specify a valid communication interface\n");
	}
	elsif(($I2C + $UART) == 2)
	{
		die("Error: Only one communication interface allowed\n");
	}

	# Check if the device is FRAM based
	if(index($device, "FR") != -1)
	{
		die("Error: MSPBoot is not compatible with FRAM devices\n       Please see MSP430FRBoot (SLAA721)\n");
	}
}

# Try to open linker and header file to confirm they exist
{
  my $fileHandle;
  
  open $fileHandle, '<', $hdr_file or die "Could not open '$hdr_file' $!\n";
  close $fileHandle;
  
  open $fileHandle, '<', $lnk_file or die "Could not open '$lnk_file' $!\n";
  close $fileHandle;
}

# Templates used to create Application and MSPBoot vector files
my $filename_tempBootVectorCCS  = "Vector_Templates/template_Boot_Vector.tmpl";
my $filename_tempAppVectorCCS  = "Vector_Templates/template_App_Vector.tmpl";

# Templates used to create Application and MSPBoot linker command files
my $filename_tempBootCCS  = "Linker_Templates/template_Boot_CCS.tmpl";
my $filename_tempAppCCS  = "Linker_Templates/template_App_CCS.tmpl";

# Templates used to create a CCS project Spec
my $filename_ProjSpecTemp  = "ProjectSpec_Templates/BootProjectSpec.tmpl";
my $filename_AppShared_ProjSpecTemp = "ProjectSpec_Templates/AppSharedCommProjectSpec.tmpl";
my $filename_AppSimple_ProjSpecTemp = "ProjectSpec_Templates/AppSimpleProjectSpec.tmpl";

# Open vector file templates
open(my $FBVT, '<', $filename_tempBootVectorCCS) or die "Could not open file '$filename_tempBootVectorCCS' $!";
open(my $FAVT, '<', $filename_tempAppVectorCCS) or die "Could not open file '$filename_tempAppVectorCCS' $!";

# Open linker file templates
open(my $FBCT, '<', $filename_tempBootCCS) or die "Could not open file '$filename_tempBootCCS' $!";
open(my $FACT, '<', $filename_tempAppCCS) or die "Could not open file '$filename_tempAppCCS' $!";

# Open project spec templates
open(my $MSPBoot_PST, '<', $filename_ProjSpecTemp) or die "Could not open file '$filename_ProjSpecTemp' $!";
open(my $AppShared_PST, '<', $filename_AppShared_ProjSpecTemp) or die "Could not open file '$filename_AppShared_ProjSpecTemp' $!";
open(my $AppSimple_PST, '<', $filename_AppSimple_ProjSpecTemp) or die "Could not open file '$filename_AppSimple_ProjSpecTemp' $!";

# Parse flash adresses from the user provided linker file
my @flash_data = getAddrLenInFile($lnk_file, "FLASH"); 				# Get address for Flash
my @flash2_data = getAddrLenInFile($lnk_file, "FLASH2"); 			# Get address for Flash2 (returns array of 0 if no upper memory)
my @RAM_data = getAddrLenInFile($lnk_file, " RAM "); 				# Get address for RAM
my @USBRAM_data = getAddrLenInFile($lnk_file, " USBRAM "); 			# Get address for USBRAM
my @vectors_data = getAddrLenInFile($lnk_file, "INT00"); 			# Get address for interrupt vectors start
my @infoA_data = getAddrLenInFile($lnk_file, "INFOA"); 				# Get address for infoA
my @infoB_data = getAddrLenInFile($lnk_file, "INFOB"); 				# Get address for infoB
my @infoC_data = getAddrLenInFile($lnk_file, "INFOC"); 				# Get address for infoC
my @infoD_data = getAddrLenInFile($lnk_file, "INFOD"); 				# Get address for infoD

my $large_mem_model = 0;
if(hex($flash2_data[1]) ne 0)
{ #large memory model device
	$large_mem_model = 1;
}

# Determine the necessary bootloader size
if(!($boot_size))
{
	if($large_mem_model & $dual_image)
	{#Dual image, large memory model
		$boot_size = "0xC00"; #3KB
	}
	elsif(!$large_mem_model & !$dual_image)
	{
		$boot_size = "0x400"  #1kB
	}
	else
	{#Every other config
		$boot_size = "0x800"; #2KB
	}
}

# Calculate flash addresses parsed from linker file
my $date = strftime('%m-%d-%Y',localtime);									# Get timestamp
my $flash_vectors_addr = hex($vectors_data[0]);				      			# Start address of interrupt vector table
my $num_vectors = (hex('FFFE') - hex($vectors_data[0]))/2;  	            # Calculate the size of the vector table
my $flash_boot_start_addr = hex('10000') - hex($boot_size);		  			# Calculate start address of boot code
my $proxy_vector_size = (hex('FFFE') - hex($vectors_data[0])) * 2;  		# Calculate the size of the proxy vector table
my $proxy_vector_addr = $flash_boot_start_addr - 2 - $proxy_vector_size;	# Calculate the proxy vector table start address
my $shared_vectors_size = 6;												# Default shared communication vector size
my $shared_vector_addr = $flash_vectors_addr -$shared_vectors_size;			# Calculate the address of the shared communication vectors
my $RAM_start_addr=hex($RAM_data[0]);										# Start address of RAM
my $RAM_end_addr=hex($RAM_data[0]) + hex($RAM_data[1]) -1;					# Calculate the RAM end address
my $variant_name = substr($device, 6);                                      # Extract device variant

if($num_shared_vectors) #if user specified a number of shared vectors, use that
{
    $shared_vectors_size = hex($num_shared_vectors)*2; 
}

# Check that the boot code will fit in available lower memory
if($flash_boot_start_addr < (hex($flash_data[0])+$proxy_vector_size))
{
    die("Error: Boot size too large to fit in lower memory.\n")
}

# Common Application files
my $app_mgr_vect_path = "Src/App/Common/TI_MSPBoot_Mgr_Vectors.h";

# Decimal representation of boot size
my $dec_boot_size = sprintf("%d", hex($boot_size));
$dec_boot_size = substr($dec_boot_size, 0, 1);

# Determine name of new project and paths
my $projDir;
my $MSPBootDir; 
my $App1Dir;
my $App2Dir;
my $filename_ProjSpecOutput;
my $filename_AppShared_ProjSpecOutput;
my $filename_AppSimple_ProjSpecOutput;
my $projectName;
my $App1ProjectName;
my $App2ProjectName;
my $filename_OutBootCCS; 
my $filename_OutApp1CCS;
my $filename_OutApp2CCS;

if($I2C & $dual_image)
{#I2C & Dual Image
    $projDir = './Generated_Output/MSPBoot_'.$variant_name.'_I2C_Dual_Img/';
	$MSPBootDir = $projDir.'MSPBoot/';
	$App1Dir = './Generated_Output/MSPBoot_'.$variant_name.'_I2C_Dual_Img/App_Simple/';
	$App2Dir = './Generated_Output/MSPBoot_'.$variant_name.'_I2C_Dual_Img/App_Shared_Comm/';
	$filename_OutBootCCS = $MSPBootDir."lnk_".$device."_I2C_".$dec_boot_size.'KB_Dual_Boot.cmd';
	$filename_OutApp1CCS = $App1Dir."lnk_".$device."_I2C_".$dec_boot_size.'KB_Dual_App.cmd';
	$filename_OutApp2CCS = $App2Dir."lnk_".$device."_I2C_".$dec_boot_size.'KB_Dual_App.cmd';
	$filename_ProjSpecOutput = $MSPBootDir.'MSPBoot_'.$variant_name.'_I2C_Dual_Img.projectspec';
	$filename_AppShared_ProjSpecOutput = $App2Dir.'App_Shared_Comm_'.$variant_name.'_I2C_Dual_Img.projectspec';
	$filename_AppSimple_ProjSpecOutput = $App1Dir.'App_Simple_'.$variant_name.'_I2C_Dual_Img.projectspec';
	$projectName = 'MSPBoot_'.$variant_name.'_I2C_Dual_Img';
	$App1ProjectName = 'App_Simple_'.$variant_name.'_I2C_Dual_Img';
	$App2ProjectName = 'App_Shared_Comm_'.$variant_name.'_I2C_Dual_Img';
}
elsif($I2C)
{#I2C & Single Image
    $projDir = './Generated_Output/MSPBoot_'.$variant_name.'_I2C_Single_Img/';
	$MSPBootDir = $projDir.'MSPBoot/';
	$App1Dir = './Generated_Output/MSPBoot_'.$variant_name.'_I2C_Single_Img/App_Simple/';
	$App2Dir = './Generated_Output/MSPBoot_'.$variant_name.'_I2C_Single_Img/App_Shared_Comm/';
	$filename_OutBootCCS = $MSPBootDir."lnk_".$device."_I2C_".$dec_boot_size.'KB_Single_Boot.cmd';
	$filename_OutApp1CCS = $App1Dir."lnk_".$device."_I2C_".$dec_boot_size.'KB_Single_App.cmd';
	$filename_OutApp2CCS = $App2Dir."lnk_".$device."_I2C_".$dec_boot_size.'KB_Single_App.cmd';
	$filename_ProjSpecOutput = $MSPBootDir.'MSPBoot_'.$variant_name.'_I2C_Single_Img.projectspec';
	$filename_AppShared_ProjSpecOutput = $App2Dir.'App_Shared_Comm_'.$variant_name.'_I2C_Single_Img.projectspec';
	$filename_AppSimple_ProjSpecOutput = $App1Dir.'App_Simple_'.$variant_name.'_I2C_Single_Img.projectspec';
	$projectName = 'MSPBoot_'.$variant_name.'_I2C_Single_Img';
	$App1ProjectName = 'App_Simple_'.$variant_name.'_I2C_Single_Img';
	$App2ProjectName = 'App_Shared_Comm_'.$variant_name.'_I2C_Single_Img';
}
elsif($UART & $dual_image)
{#UART & Dual Image
	$projDir = './Generated_Output/MSPBoot_'.$variant_name.'_UART_Dual_Img/';
	$MSPBootDir = $projDir.'MSPBoot/';
	$App1Dir = './Generated_Output/MSPBoot_'.$variant_name.'_UART_Dual_Img/App_Simple/';
	$App2Dir = './Generated_Output/MSPBoot_'.$variant_name.'_UART_Dual_Img/App_Shared_Comm/';
	$filename_OutBootCCS = $MSPBootDir."lnk_".$device."_UART_".$dec_boot_size.'KB_Dual_Boot.cmd';
	$filename_OutApp1CCS = $App1Dir."lnk_".$device."_UART_".$dec_boot_size.'KB_Dual_App.cmd';
	$filename_OutApp2CCS = $App2Dir."lnk_".$device."_UART_".$dec_boot_size.'KB_Dual_App.cmd';
	$filename_ProjSpecOutput = $MSPBootDir.'MSPBoot_'.$variant_name.'_UART_Dual_Img.projectspec';
	$filename_AppShared_ProjSpecOutput = $App2Dir.'App_Shared_Comm_'.$variant_name.'_UART_Dual_Img.projectspec';
	$filename_AppSimple_ProjSpecOutput = $App1Dir.'App_Simple_'.$variant_name.'_UART_Dual_Img.projectspec';
	$projectName = 'MSPBoot_'.$variant_name.'_UART_Dual_Img';
	$App1ProjectName = 'App_Simple_'.$variant_name.'_UART_Dual_Img';
	$App2ProjectName = 'App_Shared_Comm_'.$variant_name.'_UART_Dual_Img';
}
else
{#UART & Single Image
	$projDir = './Generated_Output/MSPBoot_'.$variant_name.'_UART_Single_Img/';
	$MSPBootDir = $projDir.'MSPBoot/';
	$App1Dir = './Generated_Output/MSPBoot_'.$variant_name.'_UART_Single_Img/App_Simple/';
	$App2Dir = './Generated_Output/MSPBoot_'.$variant_name.'_UART_Single_Img/App_Shared_Comm/';
	$filename_OutBootCCS = $MSPBootDir."lnk_".$device."_UART_".$dec_boot_size.'KB_Single_Boot.cmd';
	$filename_OutApp1CCS = $App1Dir."lnk_".$device."_UART_".$dec_boot_size.'KB_Single_App.cmd';
	$filename_OutApp2CCS = $App2Dir."lnk_".$device."_UART_".$dec_boot_size.'KB_Single_App.cmd';
	$filename_ProjSpecOutput = $MSPBootDir.'MSPBoot_'.$variant_name.'_UART_Single_Img.projectspec';
	$filename_AppShared_ProjSpecOutput = $App2Dir.'App_Shared_Comm_'.$variant_name.'_UART_Single_Img.projectspec';
	$filename_AppSimple_ProjSpecOutput = $App1Dir.'App_Simple_'.$variant_name.'_UART_Single_Img.projectspec';
	$projectName = 'MSPBoot_'.$variant_name.'_UART_Single_Img';
	$App1ProjectName = 'App_Simple_'.$variant_name.'_UART_Single_Img';
	$App2ProjectName = 'App_Shared_Comm_'.$variant_name.'_UART_Single_Img';
}

# MSPBoot sub directory paths for new project
my $BootLinkerOutputDir = $MSPBootDir ;

# Specify the name of output files for vector redirection
my $App1VectorOutputDir = $App1Dir;
my $App2VectorOutputDir = $App2Dir;

# Make directories for the new project
make_path($projDir);
make_path($MSPBootDir); 
make_path($App1VectorOutputDir); 
make_path($App2VectorOutputDir); 

# Paths to the new vector redirection files
my $filename_OutBootVectCCS = $MSPBootDir.'TI_MSPBoot_VecRed_'.$variant_name.'_Boot.c';
my $filename_OutApp1VectCCS = $App1VectorOutputDir.'TI_MSPBoot_Mgr_Vectors_'.$variant_name.'.c';
my $filename_OutApp2VectCCS = $App2VectorOutputDir.'TI_MSPBoot_Mgr_Vectors_'.$variant_name.'.c';

# Open project spec  output files
open(my $PSO, '>', $filename_ProjSpecOutput) or die "Could not open file '$filename_ProjSpecOutput' $!";
open(my $App2_PSO, '>', $filename_AppShared_ProjSpecOutput) or die "Could not open file '$filename_AppShared_ProjSpecOutput' $!";
open(my $App1_PSO, '>', $filename_AppSimple_ProjSpecOutput) or die "Could not open file '$filename_AppSimple_ProjSpecOutput' $!";

# Open vector output files
open(my $FBVO, '>', $filename_OutBootVectCCS) or die "Could not open file '$filename_OutBootVectCCS' $!";
open(my $FA1VO, '>', $filename_OutApp1VectCCS) or die "Could not open file '$filename_OutApp1VectCCS' $!";
open(my $FA2VO, '>', $filename_OutApp2VectCCS) or die "Could not open file '$filename_OutApp2VectCCS' $!";

# Open output linker command files
open(my $FBCO, '>', $filename_OutBootCCS) or die "Could not open file '$filename_OutBootCCS' $!";
open(my $FA1CO, '>', $filename_OutApp1CCS) or die "Could not open file '$filename_OutApp1CCS' $!";
open(my $FA2CO, '>', $filename_OutApp2CCS) or die "Could not open file '$filename_OutApp2CCS' $!";



my @boot_vector_text;
my @app_vector_text;

##### Replace text in Boot and App Vector files
{

    @boot_vector_text = <$FBVT>;
	@app_vector_text = <$FAVT>;
	
	# Add timestamp to boot vector file
	replacetextinBootVectorFile('<DATE>', $date);
    replacetextinBootVectorFile('<DEVICE>', $device);
	
	# Add timestamp to app vector file
	replacetextinAppVectorFile('<DATE>', $date);
    replacetextinAppVectorFile('<DEVICE>', $device);
	
	my @interrupt_data = {1,"UNUSED"};
	my $i = 0;
	my $n = 0;
	my $addr = 0;
	
	# Loop through all vectors in linker file
	while($i < $num_vectors)
	{
		# Calculate the address of the current interrupt vector in question
		$addr = hex('FFFE') - ($num_vectors * 2) + ($i*2);
		
		# Search linker file for interrupt data
		if($i < 10)
		{
			@interrupt_data = isVectReserved($lnk_file, "> INT0".$i);
		}	
		else
		{
			@interrupt_data = isVectReserved($lnk_file, "> INT".$i);
		}
		
		# Check if interrupt is populated or unused
		if($interrupt_data[0] == 0)
		{ #populated
			replacetextinBootVectorFile('<VECTORS>', "\n     APP_PROXY_VECTOR(".$n."),                        // ".gethex($addr)." = ".$interrupt_data[1]."<VECTORS>");
			replacetextinAppVectorFile('<VECTORS>', "\n    0x4030, (uint16_t) Dummy_Isr,           // APP_PROXY_VECTOR(".$n.") ".$interrupt_data[1]."<VECTORS>");
			$n++;
		}
		else
		{ #reserved
			replacetextinBootVectorFile('<VECTORS>', "\n     UNUSED,                                     // ".gethex($addr)." = unused<VECTORS>");
		}
		
		$i++;
	}
	
	# Replace the leftover <VECTORS> string
	replacetextinBootVectorFile('<VECTORS>', "");
	replacetextinAppVectorFile('<VECTORS>', "");
	
    print $FBVO @boot_vector_text ;  
    print $FA1VO @app_vector_text ;  
    print $FA2VO @app_vector_text ;	
}

close $FBVO; close $FBVT; 
close $FA1VO; close $FA2VO; close $FAVT; 


# Globals for application, download, flex, and info memory addresses
my $dual_start_addr;
my $dual_size = 0;
my $dual_end_addr;
my $appl_start_addr;
my $appl_end_addr;
my $flex_start_addr;
my $flex_end_addr;
my $flex_type;
my $info_start_addr = hex($infoD_data[0]);
my $info_end_addr;

# Find the start of info memory
if(hex($infoB_data[0]) < $info_start_addr)
{
	$info_start_addr = hex($infoB_data[0]);
}
if(hex($infoC_data[0]) < $info_start_addr)
{
	$info_start_addr = hex($infoC_data[0]);
}

# Calculate the end address of info memory
$info_end_addr = $info_start_addr + hex($infoB_data[1]) + hex($infoC_data[1]) + hex($infoD_data[1]);

# Calculate the appropriate address for the linker files
if ($dual_image)
{
	# Calculate download size and end address
    $dual_size = hex('0x10000') - hex($flash_data[0]) + hex($flash2_data[1]) - hex($boot_size); #total flash available for app + download
	
	if(hex($flash2_data[1]) eq 0) #check if small memory model device
	{
	    ########## Flash start
        # App
        ##########
        # Download
        ##########
        # Boot
        ########## 0x10000 Flash End
		
		$dual_start_addr = hex($flash_data[0]) + ($dual_size/2); # Download starts at flash start + 1/2 remaining non-boot space
		$dual_end_addr = $flash_boot_start_addr-1;				 # Download ends at the start of the boot space
		
        #flex does not exist but we will treat download area like flex space
        $flex_type = "download_no_flex";
        $appl_start_addr = hex($flash_data[0]);
        $appl_end_addr = $dual_start_addr-1;
		
		$proxy_vector_addr = $dual_start_addr - 2 - $proxy_vector_size;	# Calculate the proxy vector table start address
	}
    elsif(($dual_size/2 + hex($flash_data[0])) < $flash_boot_start_addr) # Splitting flash in half between app and download, determine if the line falls before or after boot
    {
        ########## Flash start
        # App
        ##########
        # Flex = Download
        ##########
        # Boot
        ########## 0x10000
        # Download
        ########## Flash 2 end
        
        $dual_start_addr = hex($flash_data[0]) + ($dual_size/2); 			# Download starts at flash start + 1/2 remaining non-boot space
		$dual_end_addr = hex($flash2_data[0]) + hex($flash2_data[1]) -1; 	# Download ends at the end of flash
		
        # Flex is part of download space
        $flex_type = "download";
        $flex_start_addr = $dual_start_addr;
        $flex_end_addr = $flash_boot_start_addr-1;
        $appl_start_addr = hex($flash_data[0]);
        $appl_end_addr = $flex_start_addr-1;
    }
    else
    {
        ########## Flash start
        # App
        ##########
        # Boot
        ########## 0x10000
        # Flex = App
        ##########
        # Download
        ########## Flash 2 end
        
        $dual_start_addr= hex($flash2_data[0]) + hex($flash2_data[1]) - ($dual_size/2); # Download starts at end of available flash - 1/2 remaining non-boot space
		$dual_end_addr = hex($flash2_data[0]) + hex($flash2_data[1]) -1; 				# Download ends at the end of flash
		
        # Flex is part of app space
        $flex_type = "application";
        $flex_start_addr = hex($flash2_data[0]);
        $flex_end_addr = $dual_start_addr - 1;
        $appl_start_addr = hex($flash_data[0]);
        $appl_end_addr = $flash_boot_start_addr-1;
    }
	
	# Print relevant information about boot code
	print("Application: 0x".gethex($appl_start_addr)."-0x".gethex($appl_end_addr));
	if(hex($flash2_data[1]) ne 0)
	{
		print(" & 0x".gethex($flex_start_addr)."-0x".gethex($flex_end_addr)."\n");
	}
	else
	{
	    print("\n");
	}
	print("    App_code: 0x".gethex($appl_start_addr)."-0x".gethex($proxy_vector_addr-1)."\n");
	print("    Proxy_table: 0x".gethex($proxy_vector_addr)."-0x".gethex($appl_end_addr - 2)."\n");
    print("    App_Reset_Vector: 0x".gethex($appl_end_addr - 1)."-0x".gethex($appl_end_addr)."\n");
	if(hex($flash2_data[1]) ne 0)
	{
		print("    Flex_code: 0x".gethex($flex_start_addr)."-0x".gethex($flex_end_addr)."\n");
	}
	print("Download Area: 0x".gethex($dual_start_addr)."-0x".gethex($dual_end_addr)."\n");	
	print("Bootloader Area: 0x".gethex($flash_boot_start_addr)."-0xFFFF\n");
	print("    Boot_code: 0x".gethex($flash_boot_start_addr)."-0x".gethex($shared_vector_addr-1)."\n");
	print("    Shared_Vectors: 0x".gethex($shared_vector_addr)."-0x".gethex($shared_vector_addr + $shared_vectors_size-1)."\n");
	print("    Interrupt_Vectors: 0x".gethex($flash_vectors_addr)."-0xFFFD\n");
	print("    Reset_Vector: 0xFFFE-0xFFFF\n");
}
else
{
    $appl_start_addr=hex($flash_data[0]);
    $appl_end_addr = $flash_boot_start_addr - 1;
	$flex_start_addr=hex($flash2_data[0]);
	$flex_end_addr=hex($flash2_data[0]) + hex($flash2_data[1])-1;
	
	# Print relevant information about boot code
	print("Application: 0x".gethex($appl_start_addr)."-0x".gethex($appl_end_addr));
	if(hex($flash2_data[1]) ne 0)
	{
		print(" & 0x".gethex($flex_start_addr)."-0x".gethex($flex_end_addr)."\n");
	}
	else
	{
	    print("\n");
	}
	print("    App_code: 0x".gethex($appl_start_addr)."-0x".gethex($proxy_vector_addr-1)."\n");
	print("    Proxy_table: 0x".gethex($proxy_vector_addr)."-0x".gethex($appl_end_addr - 2)."\n");
    print("    App_Reset_Vector: 0x".gethex($appl_end_addr - 1)."-0x".gethex($appl_end_addr)."\n");
	if(hex($flash2_data[1]) ne 0)
	{
		print("    Flex_code: 0x".gethex($flex_start_addr)."-0x".gethex($flex_end_addr)."\n");
	}	
	print("Bootloader Area: 0x".gethex($flash_boot_start_addr)."-0xFFFF\n");
	print("    Boot_code: 0x".gethex($flash_boot_start_addr)."-0x".gethex($shared_vector_addr-1)."\n");
	print("    Shared_Vectors: 0x".gethex($shared_vector_addr)."-0x".gethex($shared_vector_addr + $shared_vectors_size-1)."\n");
	print("    Interrupt_Vectors: 0x".gethex($flash_vectors_addr)."-0xFFFD\n");
	print("    Reset_Vector: 0xFFFE-0xFFFF\n");
}

my @boot_linker_text;
my @app_linker_text;

##### Replace text in Boot Linker command CCS file
{
    @boot_linker_text = <$FBCT>;
    replacetextinBootLinkerFile('<DATE>', $date);
    replacetextinBootLinkerFile('<DEVICE>', $device); 
    replacetextinBootLinkerFile('<RAM_START_ADDR>', gethex($RAM_start_addr));
    replacetextinBootLinkerFile('<RAM_END_ADDR>', gethex($RAM_end_addr));
    replacetextinBootLinkerFile('<RAM_PASSWD_ADDR>', gethex($RAM_start_addr+0));
    replacetextinBootLinkerFile('<RAM_STATCTRL_ADDR>', gethex($RAM_start_addr+2));
    replacetextinBootLinkerFile('<RAM_CISM_ADDR>', gethex($RAM_start_addr+3));
    replacetextinBootLinkerFile('<RAM_CALLBACKPTR_ADDR>', gethex($RAM_start_addr+4));
    replacetextinBootLinkerFile('<RAM_UNUSED_ADDR>', gethex($RAM_start_addr+8));
    replacetextinBootLinkerFile('<APPL_START_ADDR>', gethex($appl_start_addr));
    replacetextinBootLinkerFile('<APPL_END_ADDR>', gethex($appl_end_addr));
	replacetextinBootLinkerFile('<PROXY_VECTOR_ADDR>', gethex($proxy_vector_addr));
	replacetextinBootLinkerFile('<FLASH_BOOT_START_ADDR>', gethex($flash_boot_start_addr));
    replacetextinBootLinkerFile('<FLASH_VECTORS_ADDR>', gethex($flash_vectors_addr));
    replacetextinBootLinkerFile('<SHARED_VECTORS_SIZE>', $shared_vectors_size);
    replacetextinBootLinkerFile('<SHARED_VECTORS_ADDR>', gethex($shared_vector_addr));
    replacetextinBootLinkerFile('<RAM_LEN>', gethex($RAM_end_addr - ($RAM_start_addr+8)+1 ));
    replacetextinBootLinkerFile('<FLASH_BOOT_LEN>', gethex($shared_vector_addr - $flash_boot_start_addr));
    replacetextinBootLinkerFile('<FLASH_VECTORS_LEN>', gethex(0xFFFE - $flash_vectors_addr));
	
	if(hex($flash2_data[1]) ne 0)
	{
		replacetextinBootLinkerFile('<FLEX>', "\n_Flex_Start = 0x<FLEX_START_ADDR>;             /* Start of flex space (app or download, project-dependent) */\n".
									"_Flex_End = 0x<FLEX_END_ADDR>;                 /* End of flex space (app or download, project-dependent) */\n");
		replacetextinBootLinkerFile('<APP_FLEX_CRC>', "\n    _Appl_CRC_Size2 = (_Flex_End - _Flex_Start + 1); /* Number of bytes in upper memory calculated for CRC */");
	    replacetextinBootLinkerFile('<FLEX_START_ADDR>', gethex($flex_start_addr));
		replacetextinBootLinkerFile('<FLEX_END_ADDR>', gethex($flex_end_addr));
		replacetextinBootLinkerFile('<FLASH_SECTIONS>',".text       : {} >> FLASH        /* CODE                 */\n".
											 "    .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ \n".
										     "    .const      : {} >> FLASH       /* CONSTANT DATA        */"); 	
	}
	else
	{
		replacetextinBootLinkerFile('<FLEX>', "");
		replacetextinBootLinkerFile('<APP_FLEX_CRC>', "");
		replacetextinBootLinkerFile('<INFOBOOT>', "\n    INFOBOOT                : origin = 0x<BOOTINFO_START_ADDR>, length = 0x<BOOTINFO_LEN>");
	    replacetextinBootLinkerFile('<BOOTINFO_START_ADDR>', gethex($info_start_addr));
		replacetextinBootLinkerFile('<BOOTINFO_LEN>', gethex($info_end_addr - $info_start_addr));
		replacetextinBootLinkerFile('<FLASH_SECTIONS>',".text       : {} >> FLASH        /* CODE                 */\n".
											 "    .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ \n".
										     "    .const      : {} >> FLASH       /* CONSTANT DATA        */ "); 	
	}

    if($dual_image)
    {
		if($flex_type eq "download_no_flex")
		{
			# Small memory model with download area
			replacetextinBootLinkerFile('<DOWNLOAD>', "_Down_Start = 0x".gethex($dual_start_addr).
				";     /* Download Area */\n_Down_End = 0x".gethex($dual_end_addr).
				";        /*End of Download Area */\n");
			
			replacetextinBootLinkerFile('<BOOT1>', "\n/* App area     : ".gethex($appl_start_addr).'-'.gethex($appl_end_addr)."*/\n".
										   '/* Download area: '.gethex($dual_start_addr).'-'.gethex($dual_end_addr)."*/\n".
										   '/* Boot area    : '.gethex($flash_boot_start_addr).'-'.gethex(0xFFFF)."*/\n");
											   

			replacetextinBootLinkerFile('<CHECKSUMS>', "// Addresses for Dual Image support in download area \n".
										      "_Appl_Reset_Vector = (_Down_Start - 2);\n".
										      "_Down_Checksum = (_Down_Start);\n ".      
										      "_Down_Start_Memory = (_Down_Start+2);\n".
										      "_Down_CRC_Size1 = (_Down_End - _Down_Start_Memory + 1);\n".
										      "_Down_CRC_Size2 = (_Down_End - _Down_Start_Memory + 1);\n".
										      "_Down_Offset_Size = (_Down_Start-1);\n".
										      "_Down_Offset1 = (_Down_Start - _Appl_Start);\n".
										      "_Down_Offset2 = (_Down_Start - _Appl_Start);");
		}
        elsif($flex_type eq "download")
        {
            # Flex section is part of the download area
			replacetextinBootLinkerFile('<DOWNLOAD>', "_Down_Start = 0x".gethex(0x10000).
				";     /* Download Area */\n_Down_End = 0x".gethex($dual_end_addr).
				";        /*End of Download Area */\n");
            
            replacetextinBootLinkerFile('<BOOT1>', "\n/* App area     : ".gethex($appl_start_addr).'-'.gethex($appl_end_addr)."*/\n".
                                           '/* Download area: '.gethex($flex_start_addr).'-'.gethex($flex_end_addr).' & '.gethex(0x10000).'-'.gethex($dual_end_addr)."*/\n".
                                           '/* Boot area    : '.gethex($flash_boot_start_addr).'-'.gethex(0xFFFF)."*/\n");
            replacetextinBootLinkerFile('<CHECKSUMS>',"    _Appl_Reset_Vector = (_Flex_Start - 2);\n".
                                            "    _Down_Checksum = (_Flex_Start);\n".
                                            "    _Down_Start_Memory = (_Flex_Start+2);\n".
                                            "    _Down_CRC_Size1 = (_Flex_End - _Down_Start_Memory + 1);\n".
                                            "    _Down_CRC_Size2 = (_Down_End - _Down_Start + 1);\n".
                                            "    _Down_Offset_Size = (_Appl_Start +_Flex_End - _Flex_Start);\n".
                                            "    _Down_Offset1 = (_Flex_Start - _Appl_Start);\n".
                                            "    _Down_Offset2 = (_Down_Start + _Flex_Start - _Appl_Start - __Boot_Start);");       


        }
        else
        {
            # Flex section is part of the application area
            replacetextinBootLinkerFile('<DOWNLOAD>', "_Down_Start = 0x".gethex($dual_start_addr).
            ";     /* Download Area */\n_Down_End = 0x".gethex($dual_end_addr).
            ";        /*End of Download Area */\n");
            
            replacetextinBootLinkerFile('<BOOT1>', "\n/* App area     : ".gethex($appl_start_addr).'-'.gethex($appl_end_addr).' & '.gethex($flex_start_addr).'-'.gethex($flex_end_addr)."*/\n".
                                   '/* Download area: '.gethex($dual_start_addr).'-'.gethex($dual_end_addr)."*/\n".
                                   '/* Boot area    : '.gethex($flash_boot_start_addr).'-'.gethex(0xFFFF)."*/\n");
            replacetextinBootLinkerFile('<CHECKSUMS>',"    _Appl_Reset_Vector = (__Boot_Start - 2);\n".
                                            "    _Down_Checksum = (_Down_Start);\n".
                                            "    _Down_Start_Memory = (_Down_Start+2);\n".
                                            "    _Down_CRC_Size1 = (_Down_End - _Down_Start_Memory + 1);\n".
                                            "    _Down_CRC_Size2 = (_Down_End - _Down_Start_Memory + 1);\n".
                                            "    _Down_Offset_Size = (__Boot_Start - 1);\n".
                                            "    _Down_Offset1 = (_Down_Start - _Appl_Start);\n".
                                            "    _Down_Offset2 = (__Boot_Start - _Appl_Start + _Down_Start - _Flex_Start);");                      
        
        }        
    }
    else
    {
		replacetextinBootLinkerFile('<DOWNLOAD>',"");
		replacetextinBootLinkerFile('<CHECKSUMS>',"    _Appl_Reset_Vector = (__Boot_Start - 2);");
		replacetextinBootLinkerFile('<BOOT1>', "");
    }

    print $FBCO @boot_linker_text ;    
}

##### Replace text in App linker command CCS file
{
    @app_linker_text = <$FACT>;
	
    replacetextinAppLinkerFile('<DATE>', $date);
    replacetextinAppLinkerFile('<DEVICE>', $device);
    replacetextinAppLinkerFile('<RAM_START_ADDR>', gethex($RAM_start_addr));
    replacetextinAppLinkerFile('<RAM_END_ADDR>', gethex($RAM_end_addr));
    replacetextinAppLinkerFile('<RAM_PASSWD_ADDR>', gethex($RAM_start_addr+0));
    replacetextinAppLinkerFile('<RAM_STATCTRL_ADDR>', gethex($RAM_start_addr+2));
    replacetextinAppLinkerFile('<RAM_CISM_ADDR>', gethex($RAM_start_addr+3));
    replacetextinAppLinkerFile('<RAM_CALLBACKPTR_ADDR>', gethex($RAM_start_addr+4));
    replacetextinAppLinkerFile('<RAM_UNUSED_ADDR>', gethex($RAM_start_addr+8));
    replacetextinAppLinkerFile('<APPL_START_ADDR>', gethex($appl_start_addr));
    replacetextinAppLinkerFile('<FLASH_BOOT_START_ADDR>', gethex($flash_boot_start_addr));
    replacetextinAppLinkerFile('<FLASH_VECTORS_ADDR>', gethex($flash_vectors_addr));
    replacetextinAppLinkerFile('<SHARED_VECTORS_SIZE>', $shared_vectors_size);
    replacetextinAppLinkerFile('<SHARED_VECTORS_ADDR>', gethex($shared_vector_addr));
    replacetextinAppLinkerFile('<APP_FLASH_LEN>', gethex($proxy_vector_addr-($appl_start_addr+2)));
    replacetextinAppLinkerFile('<PROXY_VECTOR_SIZE>', ($proxy_vector_size));
    replacetextinAppLinkerFile('<PROXY_VECTOR_ADDR>', gethex($proxy_vector_addr));
    replacetextinAppLinkerFile('<RAM_LEN>', gethex($RAM_end_addr - ($RAM_start_addr+8)+1 ));
    replacetextinAppLinkerFile('<APP_FLASH_START>', gethex($appl_start_addr+2));
    replacetextinAppLinkerFile('<APP_RESET_VECTOR>', gethex($appl_end_addr - 1));

    if($dual_image)
    {
		if($flex_type eq "download_no_flex")
		{
			replacetextinAppLinkerFile('<BOOT1>', "\n/* App area     : ".gethex($appl_start_addr).'-'.gethex($appl_end_addr)."*/\n".
										   '/* Download area: '.gethex($dual_start_addr).'-'.gethex($dual_end_addr)."*/\n".
										   '/* Boot area    : '.gethex($flash_boot_start_addr).'-'.gethex(0xFFFF)."*/\n");
			replacetextinAppLinkerFile('<APP_FLASH2_START>', "");
            replacetextinAppLinkerFile('<FLASH_SECTIONS>',   ".text       : {} >> FLASH               /* CODE                 */\n".
                                                    "    .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ \n".
                                                    "    .const      : {} >> FLASH       /* CONSTANT DATA        */"); 			
		}
        elsif($flex_type eq "download")
        {
            replacetextinAppLinkerFile('<BOOT1>', "\n/* App area     : ".gethex($appl_start_addr).'-'.gethex($appl_end_addr)."*/\n".
                                           '/* Download area: '.gethex($flex_start_addr).'-'.gethex($flex_end_addr).' & '.gethex(0x10000).'-'.gethex($dual_end_addr)."*/\n".
                                           '/* Boot area    : '.gethex($flash_boot_start_addr).'-'.gethex(0xFFFF)."*/\n");
            
            replacetextinAppLinkerFile('<APP_FLASH2_START>', "");
            replacetextinAppLinkerFile('<FLASH_SECTIONS>',   ".text       : {} >> FLASH               /* CODE                 */\n".
                                                    "    .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ \n".
                                                    "    .const      : {} >> FLASH       /* CONSTANT DATA        */");    
        }
        else
        {          
            replacetextinAppLinkerFile('<BOOT1>', "\n/* App area     : ".gethex($appl_start_addr).'-'.gethex($appl_end_addr).' & '.gethex($flex_start_addr).'-'.gethex($flex_end_addr)."*/\n".
                                   '/* Download area: '.gethex($dual_start_addr).'-'.gethex($dual_end_addr)."*/\n".
                                   '/* Boot area    : '.gethex($flash_boot_start_addr).'-'.gethex(0xFFFF)."*/\n");
                                   
            replacetextinAppLinkerFile('<APP_FLASH2_START>', "\n    FLASH2                  : origin = 0x".gethex($flex_start_addr).', length = 0x'.gethex($flex_end_addr - $flex_start_addr + 1));     
            replacetextinAppLinkerFile('<FLASH_SECTIONS>',   "#ifndef __LARGE_CODE_MODEL__\n".
                                                    "        .text       : {} >> FLASH               /* CODE                 */\n".
                                                    "    #else \n".
                                                    "        .text       : {} >> FLASH | FLASH2      /* CODE                 */ \n".
                                                    "    #endif \n\n".
                                                    "        .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ \n".
                                                    "    #ifndef __LARGE_DATA_MODEL__ \n".
                                                    "        .const      : {} >> FLASH       /* CONSTANT DATA        */ \n".
                                                    "    #else \n".
                                                    "        .const      : {} >> FLASH2 | FLASH    /* CONSTANT DATA        */ \n".
                                                    "    #endif \n");    
        }
    }
    else
    {
        replacetextinAppLinkerFile('<BOOT1>', "");
		if(hex($flash2_data[1]) ne 0)
		{
			replacetextinAppLinkerFile('<APP_FLASH2_START>',"\n    FLASH2                  : origin = 0x".gethex($flex_start_addr).', length = 0x'.gethex($flex_end_addr - $flex_start_addr + 1));
			replacetextinAppLinkerFile('<FLASH_SECTIONS>',   "#ifndef __LARGE_CODE_MODEL__\n".
                                                    "        .text       : {} >> FLASH               /* CODE                 */\n".
                                                    "    #else \n".
                                                    "        .text       : {} >> FLASH | FLASH2      /* CODE                 */ \n".
                                                    "    #endif \n\n".
                                                    "        .cinit      : {} > FLASH        /* INITIALIZATION TABLES*/ \n".
                                                    "    #ifndef __LARGE_DATA_MODEL__ \n".
                                                    "        .const      : {} >> FLASH       /* CONSTANT DATA        */ \n".
                                                    "    #else \n".
                                                    "        .const      : {} >> FLASH2 | FLASH    /* CONSTANT DATA        */ \n".
                                                    "    #endif \n");
		}
		else
		{
			replacetextinAppLinkerFile('<APP_FLASH2_START>', "");
			replacetextinAppLinkerFile('<FLASH_SECTIONS>', ".text       : {} >> FLASH               /* CODE                 */\n".
												  "    .cinit      : {} > FLASH   /* INITIALIZATION TABLES*/\n".
												  "    .const      : {} >> FLASH   /* CONSTANT DATA        */");
		}
    }
	
	#Fill in Infomem sections
	if(hex($infoA_data[1]) ne 0)
	{
		replacetextinAppLinkerFile('<INFOA>', "INFOA                   : origin = 0x".gethex(hex($infoA_data[0])).", length = 0x".gethex(hex($infoA_data[1]))."\n");
		replacetextinAppLinkerFile('<INFOA_SECTION>', ".infoA     : {} > INFOA              /* MSP430 INFO FLASH Memory segments */\n");
	}
	else
	{
		replacetextinAppLinkerFile('<INFOA>', "");
		replacetextinAppLinkerFile('<INFOA_SECTION>',"");
	}
	if(hex($infoB_data[1]) ne 0)
	{
		replacetextinAppLinkerFile('<INFOB>', "    INFOB                   : origin = 0x".gethex(hex($infoB_data[0])).", length = 0x".gethex(hex($infoB_data[1]))."\n");
		replacetextinAppLinkerFile('<INFOB_SECTION>', "    .infoB     : {} > INFOB\n");
	}
	else
	{
		replacetextinAppLinkerFile('<INFOB>', "");
		replacetextinAppLinkerFile('<INFOB_SECTION>',"");
	}
	if(hex($infoC_data[1]) ne 0)
	{
		replacetextinAppLinkerFile('<INFOC>', "    INFOC                   : origin = 0x".gethex(hex($infoC_data[0])).", length = 0x".gethex(hex($infoC_data[1]))."\n");
		replacetextinAppLinkerFile('<INFOC_SECTION>', "    .infoC     : {} > INFOC\n");
	}
	else
	{
		replacetextinAppLinkerFile('<INFOB>', "");
		replacetextinAppLinkerFile('<INFOB_SECTION>',"");
	}
	if(hex($infoD_data[1]) ne 0)
	{
		replacetextinAppLinkerFile('<INFOD>', "    INFOD                   : origin = 0x".gethex(hex($infoD_data[0])).", length = 0x".gethex(hex($infoD_data[1])));
		replacetextinAppLinkerFile('<INFOD_SECTION>', "    .infoD     : {} > INFOD");
	}
	else
	{
		replacetextinAppLinkerFile('<INFOD>', "");
		replacetextinAppLinkerFile('<INFOD_SECTION>',"");
	}
	
	#Fill in USBRAM section
	if(hex($USBRAM_data[1]) ne 0)
	{
		replacetextinAppLinkerFile('<USBRAM>', "\n    USBRAM                  : origin = 0x".gethex(hex($USBRAM_data[0])).", length = 0x".gethex(hex($USBRAM_data[1])));
	}
	else
	{
		replacetextinAppLinkerFile('<USBRAM>', "");
	}

    print $FA1CO @app_linker_text ;  
    print $FA2CO @app_linker_text ;  	
}

close $FBCO; close $FBCT; 
close $FA1CO; close $FA2CO; close $FACT; 

my @ps_vector_text;
my @ps_App1_vector_text;
my @ps_App2_vector_text;
##### Create a MSPBoot, App1 and App2 CCS project spec file
{
	@ps_vector_text = <$MSPBoot_PST>;
	@ps_App1_vector_text = <$AppSimple_PST>;
	@ps_App2_vector_text = <$AppShared_PST>;
	
	#MSPBoot Project name
	replacetextinProjectSpecFile('<DEVICE>', $device);
	replacetextinProjectSpecFile('<PROJECT_NAME>', $projectName);
	
	#App1 Project Name
	replacetextinApp1ProjectSpecFile('<DEVICE>', $device);
	replacetextinApp1ProjectSpecFile('<PROJECT_NAME>', $App1ProjectName);
	
	#App2 Project Name
	replacetextinApp2ProjectSpecFile('<DEVICE>', $device);
	replacetextinApp2ProjectSpecFile('<PROJECT_NAME>', $App2ProjectName);
	
	if($large_mem_model)
	{ #large memory model
		#MSPBoot
		replacetextinProjectSpecFile('<CODE_MODEL>', "large");
		replacetextinProjectSpecFile('<DATA_MODEL>', "large");
		
		#App1
		replacetextinApp1ProjectSpecFile('<CODE_MODEL>', "large");
		replacetextinApp1ProjectSpecFile('<DATA_MODEL>', "large");
		
		#App2
		replacetextinApp2ProjectSpecFile('<CODE_MODEL>', "large");
		replacetextinApp2ProjectSpecFile('<DATA_MODEL>', "large");
	}
	else
	{#small memory model
	    #MSPBoot
		replacetextinProjectSpecFile('<CODE_MODEL>', "small");
		replacetextinProjectSpecFile('<DATA_MODEL>', "small");
		
		#App1
		replacetextinApp1ProjectSpecFile('<CODE_MODEL>', "small");
		replacetextinApp1ProjectSpecFile('<DATA_MODEL>', "small");
		
		#App2
		replacetextinApp2ProjectSpecFile('<CODE_MODEL>', "small");
		replacetextinApp2ProjectSpecFile('<DATA_MODEL>', "small");
	}
	
	
	# MSPBoot Directory
	replacetextinProjectSpecFile('<BOOT_PATH>', "../../../Src/MSPBoot/boot.c");
	replacetextinProjectSpecFile('<MAIN_PATH>', "../../../Src/MSPBoot/main.c");
	replacetextinProjectSpecFile('<COMMON_PATH>', "../../../Src/MSPBoot/TI_MSPBoot_Common.h");
	replacetextinProjectSpecFile('<CONFIG_PATH>', "../../../Src/MSPBoot/TI_MSPBoot_Config.h");
	replacetextinProjectSpecFile('<LINKER_PATH>', "../../../".$filename_OutBootCCS);
	
	#App1 Directory
	replacetextinApp1ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Simple/main.c");
	replacetextinApp1ProjectSpecFile('<VECREDH_PATH>', "../../../Src/App/Common/TI_MSPBoot_Mgr_Vectors.h");
	replacetextinApp1ProjectSpecFile('<VECRED_PATH>', "../../../".$filename_OutApp1VectCCS);
	replacetextinApp1ProjectSpecFile('<LINKER_PATH>', "../../../".$filename_OutApp1CCS);
	
	#App2 Directory
	replacetextinApp2ProjectSpecFile('<VECREDH_PATH>', "../../../Src/App/Common/TI_MSPBoot_Mgr_Vectors.h");
	replacetextinApp2ProjectSpecFile('<VECRED_PATH>', "../../../".$filename_OutApp2VectCCS);
	replacetextinApp2ProjectSpecFile('<LINKER_PATH>', "../../../".$filename_OutApp2CCS);
	
	# AppMgr Directory
	replacetextinProjectSpecFile('<CRC_PATH>', "../../../Src/MSPBoot/AppMgr/Common/Crc.c");
	replacetextinProjectSpecFile('<CRCH_PATH>', "../../../Src/MSPBoot/AppMgr/Common/Crc.h");
	replacetextinProjectSpecFile('<APPMGRH_PATH>', "../../../Src/MSPBoot/AppMgr/Common/TI_MSPBoot_AppMgr.h");
	replacetextinProjectSpecFile('<VECRED_PATH>', "../../../".$filename_OutBootVectCCS);
	
	if($dual_image & $large_mem_model)
	{ # dual image and large memory model
		replacetextinProjectSpecFile('<APPMGR_PATH>', "../../../Src/MSPBoot/AppMgr/TI_MSPBoot_AppMgrDualImg_20Bit.c");
	}
	elsif($dual_image & !$large_mem_model)
	{ # dual image and small memory model
	    replacetextinProjectSpecFile('<APPMGR_PATH>', "../../../Src/MSPBoot/AppMgr/TI_MSPBoot_AppMgrDualImg_16Bit.c");
	}
	else
	{ # single image (both memory models)
	    replacetextinProjectSpecFile('<APPMGR_PATH>', "../../../Src/MSPBoot/AppMgr/TI_MSPBoot_AppMgr.c");
	}
	
	# Comm Directory
	replacetextinProjectSpecFile('<PHYDLH_PATH>', "../../../Src/MSPBoot/Comm/Common/TI_MSPBoot_CI_PHYDL.h");
	replacetextinProjectSpecFile('<CIH_PATH>', "../../../Src/MSPBoot/Comm/Common/TI_MSPBoot_CI.h");
	replacetextinProjectSpecFile('<NWKH_PATH>', "../../../Src/MSPBoot/Comm/Common/TI_MSPBoot_CI_NWK_APP.h");
	if($I2C)
	{ #I2C communications interface
		if(searchFile($hdr_file,"__MSP430_HAS_EUSCI"))
		{# Device has eUSCI module
			replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/I2C/eUSCI/PHY_DL/TI_MSPBoot_CI_PHYDL_eUSCI_I2C_slave.c");
			if($large_mem_model)
			{ # I2C eUSCI large mem model
				replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/I2C/eUSCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_I2C_20Bit.c");
			}
			else
			{ #I2C eUSCI small mem model
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/I2C/eUSCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_I2C_16Bit.c");
			}
			
			#App2 Directory
			replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/I2C/eUSCI/main.c");
			replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/I2C/eUSCI/TI_MSPBoot_Mgr_CI_eUSCI_I2C.c");
			replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/I2C/eUSCI/TI_MSPBoot_Mgr_CI_eUSCI_I2C.h");
			
		}
		elsif(searchFile($hdr_file,"__MSP430_HAS_USCI"))
		{# Device has USCI module
			if($large_mem_model)
			{ # I2C USCI large mem model
				replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/I2C/USCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_I2C_20Bit.c");
			}
			else
			{ # I2C eUSCI small mem model
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/I2C/USCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_I2C_16Bit.c");
			}
			
			# Check if device s x2xx variant (different IFG registers from normal USCI)
			if(index($variant_name, "2") == 1)
			{# Device is x2xx variant (IFG2 used for comm interrupts)
			    replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/I2C/USCI/PHY_DL/TI_MSPBoot_CI_PHYDL_USCI_I2C_Slave_x2xx.c");
				
				#App2 Directory
				replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/I2C/USCI/main.c");
				replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/I2C/USCI/TI_MSPBoot_Mgr_CI_USCI_I2C_x2xx.c");
				replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/I2C/USCI/TI_MSPBoot_Mgr_CI_USCI_I2C.h");
			}
			else
			{ # Device is NOT an x2xx variant
			    replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/I2C/USCI/PHY_DL/TI_MSPBoot_CI_PHYDL_USCI_I2C_slave.c");
				
				#App2 Directory
				replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/I2C/USCI/main.c");
				replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/I2C/USCI/TI_MSPBoot_Mgr_CI_USCI_I2C.c");
				replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/I2C/USCI/TI_MSPBoot_Mgr_CI_USCI_I2C.h");
			
			}
		}
		elsif(searchFile($hdr_file,"__MSP430_HAS_USI"))
		{# Device has USI module
			replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/I2C/USI/PHY_DL/TI_MSPBoot_CI_PHYDL_USI_I2C_slave.c");
			if($large_mem_model)
			{# I2C USCI large mem model
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/I2C/USI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_I2C_20Bit.c");
			}
			else
			{# I2C USCI small mem model
				replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/I2C/USI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_I2C_16Bit.c");
			}
			
			#App2 Directory
			replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/I2C/USI/main.c");
			replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/I2C/USI/TI_MSPBoot_Mgr_CI_USI_I2C.c");
			replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/I2C/USI/TI_MSPBoot_Mgr_CI_USI_I2C.h");
		}
		else
		{
		    close $PSO; close $MSPBoot_PST; close $App2_PSO; close $App1_PSO;
			remove_tree($projDir);
			die("Error: I2C communication not possible on this device\n");
		}
	}
	else
	{ #UART
		if(searchFile($hdr_file,"__MSP430_HAS_EUSCI"))
		{# Device has eUSCI module
			replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/UART/eUSCI/PHY_DL/TI_MSPBoot_CI_PHYDL_eUSCI_UART.c");
			if($large_mem_model)
			{# UART eUSCI large mem model
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/UART/eUSCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_UART_20Bit.c");
			}
			else
			{# UART eUSCI small mem model
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/UART/eUSCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_UART_16Bit.c");
			}
			
			#App2 Directory
			replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/UART/eUSCI/main.c");
			replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/UART/eUSCI/TI_MSPBoot_Mgr_CI_eUSCI_UART.c");
			replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/UART/eUSCI/TI_MSPBoot_Mgr_CI_eUSCI_UART.h");
		}
		elsif(searchFile($hdr_file,"__MSP430_HAS_USCI"))
		{# Device has USCI module
			if($large_mem_model)
			{
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/UART/USCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_UART_20Bit.c");
			}
			else
			{
			    replacetextinProjectSpecFile('<NWK_PATH>', "../../../Src/MSPBoot/Comm/UART/USCI/NWK_APP/TI_MSPBoot_CI_NWK_APP_BSL_UART_16Bit.c");
			}
			
			if(index($variant_name, "2") == 1)
			{# Device is x2xx variant (IFG2 used for comm interrupts)
			    replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/UART/USCI/PHY_DL/TI_MSPBoot_CI_PHYDL_USCI_UART_x2xx.c");
			
				#App2 Directory
				replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/UART/USCI/main.c");
				replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/UART/USCI/TI_MSPBoot_Mgr_CI_USCI_UART_x2xx.c");
				replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/UART/USCI/TI_MSPBoot_Mgr_CI_USCI_UART.h");
			}
			else
			{ # Device is NOT an x2xx variant
			    replacetextinProjectSpecFile('<PHYDL_PATH>', "../../../Src/MSPBoot/Comm/UART/USCI/PHY_DL/TI_MSPBoot_CI_PHYDL_USCI_UART.c");
			
				#App2 Directory
				replacetextinApp2ProjectSpecFile('<MAIN_PATH>', "../../../Src/App/App_Shared_Comm/UART/USCI/main.c");
				replacetextinApp2ProjectSpecFile('<CI_PATH>',  "../../../Src/App/App_Shared_Comm/UART/USCI/TI_MSPBoot_Mgr_CI_USCI_UART.c");
				replacetextinApp2ProjectSpecFile('<CIH_PATH>', "../../../Src/App/App_Shared_Comm/UART/USCI/TI_MSPBoot_Mgr_CI_USCI_UART.h");
			}
		}
		else
		{
			close $PSO; close $MSPBoot_PST; close $App2_PSO; close $App1_PSO;
			remove_tree($projDir);
			die("Error: UART communication not possible on this device\n");
		}
	}

	# MI Directory
	if($large_mem_model)
	{# 20 bit (large mem model)
		replacetextinProjectSpecFile('<MIH_PATH>', "../../../Src/MSPBoot/MI/20_bit/TI_MSPBoot_MI_20Bit.h");
		
		if(searchFile($hdr_file,"__MSP430_HAS_FLASH2__"))
		{#MSP430 flash controller needs a source clock
			if($dual_image)
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/20_bit/Flash2/TI_MSPBoot_MI_FlashDualImg_20Bit.c");
			}
			else
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/20_bit/Flash2/TI_MSPBoot_MI_Flash_20Bit.c");
			}
		}
		else
		{#MPS430 flash controller has its own clock generator
			if($dual_image)
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/20_bit/Flash1/TI_MSPBoot_MI_FlashDualImg_20Bit.c");
			}
			else
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/20_bit/Flash1/TI_MSPBoot_MI_Flash_20Bit.c");
			}
		}
	}
	else
	{ # 16 bit (small mem model)
	    replacetextinProjectSpecFile('<MIH_PATH>', "../../../Src/MSPBoot/MI/16_bit/TI_MSPBoot_MI_16Bit.h");
		
		if(searchFile($hdr_file,"__MSP430_HAS_FLASH2__"))
		{#MSP430 flash controller needs a source clock
			if($dual_image)
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/16_bit/Flash2/TI_MSPBoot_MI_FlashDualImg_16Bit.c");
			}
			else
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/16_bit/Flash2/TI_MSPBoot_MI_Flash_16Bit.c");
			}
		}
		else
		{#MPS430 flash controller has its own clock generator
			if($dual_image)
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/16_bit/Flash1/TI_MSPBoot_MI_FlashDualImg_16Bit.c");
			}
			else
			{
				replacetextinProjectSpecFile('<MI_PATH>', "../../../Src/MSPBoot/MI/16_bit/Flash1/TI_MSPBoot_MI_Flash_16Bit.c");
			}
		}
	}
    print $PSO @ps_vector_text ; 
    print $App2_PSO @ps_App2_vector_text ;
    print $App1_PSO @ps_App1_vector_text ;   	
    close $PSO; close $MSPBoot_PST; close $App2_PSO; close $App1_PSO;
}

#param 0 = file
#param 1 = text to find
#returns(1,"UNUSED") if interrupt vector is reserved
#returns(0,"Vector Name") if interrupt vector is populated
sub isVectReserved{
    my ($file, $search_str) = @_;
    my @val = (1,"UNUSED");
	
	open my $fh, '<', $file or die "Could not open '$file' $!\n";

    while (my $line = <$fh>) #go through one line at a time
    {
        chomp $line;
    
       # find the line containing the interrupt vector section definition
       if(($line =~ /$search_str/))
       {
			if(index($line, "VECT_INIT") > 0)
			{
			    my $temp = index($line, ":");
				$val[0] = 0;
				$val[1] = substr($line, 3, ($temp - 4));
			}
            
            last;
       }
    }
	
	return @val;
}

#param 0 = text to replace
#param 1 = new text
sub replacetextinBootLinkerFile{
    my ($in_text, $out_text) = @_;
    for (@boot_linker_text)
    {
        s/$in_text/$out_text/g;
    }
}
#param 0 = text to replace
#param 1 = new text
sub replacetextinAppLinkerFile{
    my ($in_text, $out_text) = @_;
    for (@app_linker_text)
    {
        s/$in_text/$out_text/g;
    }
}
#param 0 = text to replace
#param 1 = new text
sub replacetextinBootVectorFile{
    my ($in_text, $out_text) = @_;
    for (@boot_vector_text)
    {
        s/$in_text/$out_text/g;
    }
}
#param 0 = text to replace
#param 1 = new text
sub replacetextinAppVectorFile{
    my ($in_text, $out_text) = @_;
    for (@app_vector_text)
    {
        s/$in_text/$out_text/g;
    }
}
#param 0 = text to replace
#param 1 = new text
sub replacetextinProjectSpecFile{
    my ($in_text, $out_text) = @_;
    for (@ps_vector_text)
    {
        s/$in_text/$out_text/g;
    }
}
#param 0 = text to replace
#param 1 = new text
sub replacetextinApp1ProjectSpecFile{
    my ($in_text, $out_text) = @_;
    for (@ps_App1_vector_text)
    {
        s/$in_text/$out_text/g;
    }
}
#param 0 = text to replace
#param 1 = new text
sub replacetextinApp2ProjectSpecFile{
    my ($in_text, $out_text) = @_;
    for (@ps_App2_vector_text)
    {
        s/$in_text/$out_text/g;
    }
}

#param 0 = text to find
#param 1 = file
#returns(0,0) if info not found in file
sub getAddrLenInFile{
    my ($file, $search_str) = @_;
    my @val = (0,0);
	
    open my $fh, '<', $file or die "Could not open '$file' $!\n";

    while (my $line = <$fh>) #go through one line at a time
    {
        chomp $line;
    
    # find the line containing the memory section definition we are looking for
       if(($line =~ /$search_str/) && ($line =~ "origin"))
       {
       # parse the address
            my $temp = index($line, ': origin = ');
            my $temp2 = index($line, ',');
            $val[0] = substr($line, $temp + 11, ($temp2 - ($temp + 11)));

        #parse the length    
            $temp = index($line, 'length = ');
            $val[1] = substr($line, $temp + 9);
            
            last;
       }
    }
	
    return @val;
}

#param 0 = text to find
#param 1 = file
#returns 0 if info not found, 1 if found in file
sub searchFile{
    my ($file, $search_str) = @_;
    my $val = 0;
	
    open my $fh, '<', $file or die "Could not open '$file' $!\n";

    while (my $line = <$fh>) #go through one line at a time
    {
        chomp $line;
    
    # find the line containing the memory section definition we are looking for
       if(($line =~ /$search_str/))
       {
            $val = 1;
            last;
       }
    }
	
	close $fh;
	
    return $val;
}

#param 0 = text to find
#param 1 = file
sub getdeviceinFile{
    my ($file) = @_;
    my $val = 0;
    open my $fh, '<', $file or die "Could not open '$file' $!\n";

    while (my $line = <$fh>) #go through one line at a time
    {
        chomp $line;
    
       if($line =~ /-l/)
       {
            my $temp = index($line, '-l ');
            my $temp2 = index($line, '.cmd');
            $val = substr($line, $temp + 3, ($temp2 - ($temp + 3)));
            last;
       }
    }
    return $val;
}

sub gethex{
    return sprintf("%X", $_[0]);
}

#Help 
sub help{ 
    print "
================================================================================
    Generates project files for MSPBoot for Flash Devices
================================================================================
[] denotes optional field
ALL NUMBERS SHOULD BE HEX NOT DECIMAL
MSPBootProjectCreator.pl   
                      [-help] 
                      -hdr <header_file>
                      -lnk_file <lnk_msp430.cmd>
                      [-boot_size <size>]
                      [-shared_vectors <number>] 
                      [-dual_image]
                      [-I2C]
                      [-UART]
        
  -hdr              Specifies the header file for the MSP430 device variant 
   <header_file>    Default header file for the device being used. 
                    Must be in same directory or specify a path.
  -lnk_file         Specifies the default linker file for the device being used. 
  <lnk_msp430.cmd>  Default linker file for the device being used. 
                    Must be in same directory or specify a path.
  -boot_size        Optional commands that specifies size of bootloader area.
                    If omitted, the default bootloader size will be used.  
  <size>            Size of bootloader. (Only increments of 0x400 allowed).
  -shared_vectors   Specifies the number of shared vectors. If not specified,
                    default value is 3 vectors. 
  <number>          Number of shared vectors (in hex)
  -dual_image       If enabled, the output files include definitions supporting
                    dual image. Otherwise, single image mode assumed.
  -I2C              If enabled, the output files include definitions supporting
                    I2C communication interface.
  -UART             If enabled, the output files include definitions supporting
                    UART communication interface.				
  -help             Displays this help dialog. 
================================================================================
See the MSPBoot User's Guide for more information.
================================================================================
EXAMPLE: 
MSP430F5529 3kB I2C bootloader with dual image support. 

The memory map to be generated is as follows:

RAM: 0x2400-0x43FF
Application: 0x4400-0xF3FF
    App_code: 0x4400-0xF301
    Proxy_Table: 0xF302-0xF3FD
    APP_Reset_vector: 0xF3FE-0xF3FF
Download Area: 0x14A00-243FF
Bootloader: 0xF400-0xFFFF
    Boot_Code: 0xF400-0xFF79
    Shared_vectors: 0xFF7A-0xFF7F
    Vectors: 0xFF80-0xFFFD
    Reset_Vectors: 0xFFFE-0xFFFF

This command generates the necessary files for an MSP430F5529 based MSPBoot
    
perl MSPBootProjectCreator.pl -hdr msp430f5529.h -lnk_file lnk_msp430f5529.cmd 
     -I2C -dual_image

"; 
    exit; 
} 
