/*
 * \file   TI_MSPBoot_CI_PHYDL_USI_I2C_slave.c
 *
 * \brief  Driver for I2C Slave Physical-DataLink layers using USI
 *
 */
/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

//
// Include files
//
#include "msp430.h"
#include "TI_MSPBoot_Common.h"
#include "TI_MSPBoot_CI.h"
#include "TI_MSPBoot_AppMgr.h"
#include "TI_MSPBoot_CI_PHYDL.h"

//TODO: Change I2C registers if necessary
#define I2C_CTL0_REG  		USICTL0
#define I2C_CTL1_REG        USICTL1
#define I2C_CKCTL_REG       USICKCTL                       
#define I2C_CNT_REG		    USICNT 			 	
#define I2C_SHIFT_REG_L	    USISRL
#define I2C_SHIFT_REG_H	    USISRH		 

//TODO: Timer registers if necessary
#define TIMER_CTL_REG       TACTL
#define TIMER_SRC_SEL       TASSEL_2  //SMCLK
#define TIMER_CLR           TACLR
#define TIMER_CCR_REG       TACCR0
#define TIMER_CCTL_REG      TACCTL0

//
//  Type Definitions
//
/*! State machine for USI.*/
typedef enum {
    USI_STATE_IDLE=0,           /*! Initialized state waiting for start */
    USI_STATE_START_RECEIVED,   /*! Start bit received */
    USI_STATE_TX_ACK_ADDR,      /*! Send ACK/NACK after address */
    USI_STATE_RXBYTE,           /*! Byte reception */
    USI_STATE_TX_ACK_DATA,      /*! Send ACK after byte RX */
    USI_STATE_TXBYTE,           /*! Byte Transmission */
    USI_STATE_RX_ACK_DATA,      /*! Wait for ACK/NACK after byte TX */
    USI_STATE_TEST_ACK,         /*! Check ACK/NACK after byte TX */
    USI_STATE_PREP_RESTART      /*! Transfer end, wait for restart */
}USI_State_Machine_e;


//
//  Global variables
//
//    Note that these variables are assigned to a specific location in RAM
//    since they can also be used during application.
//    If not used by application, the location doesn't matter.
//
/*! Pointer to the Communication Interface callback structure 
 *   The NWK layer will define its callback structure and pass the pointer to
 *   this layer. An application can also declare its own structure and pass it
 *   to this layer in order to use the same interface.
 */
/*! Comm Interface callback structure */
extern t_CI_Callback  * CI_Callback_ptr;

/*! State machine used by this interface. 
 */
extern USI_State_Machine_e CI_State_Machine;

//
//  Local definitions
//
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
/*! Number of clock cycles for a millisecond. Clock runs from SMCLK/4=MCLK/4 */
#define MS_CYCLES                   ((MCLK/4)/1000)
/*! Number of clock cycles for a SMBus timeout (25 ms) */
#define TIMEOUT_CYCLES        (25*MS_CYCLES)
#endif

//
//  Local Function prototypes
//
//
//  Local function prototypes
//
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
static void Timer_Timeout_Init(void);
static void Timer_Timeout_Start(void);
static void Timer_Timeout_Stop(void);
static uint8_t Timer_Timeout_Expired(void);
#endif
#ifdef CONFIG_CI_PHYDL_STOP_CALLBACK
static void Check_StopFlag(void);
#endif

//
//  Function definitions
//
/******************************************************************************
*
 * @brief   Initializes the USI I2C Slave interface
 *  - Sets corresponding GPIOs for I2C functionality
 *  - Resets and then configures the I2C
 *  - Initializes the I2C state machine
 *  - Initializes the callbacks
 *  The NWK layer will define its callback structure and pass the pointer to
 *   this function. An application can also declare its own structure and call
 *   this function in order to use the same interface.
 *
 *  @param  CI_Callback     Pointer to Communication interface callbacks
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_Init(t_CI_Callback * CI_Callback)
{
   // P1SEL |= (BIT6 | BIT7);                 // Enable P1[6:7] for USI I2C mode

    I2C_CTL0_REG = (USIPE6+USIPE7+USISWRST);     // USI in reset, SCL/SDA enabled, I2C slave
    I2C_CTL1_REG = (USII2C);                     // Enable I2C mode, interrupt disabled
                                            // Start and counter
    I2C_CKCTL_REG = (USICKPL);                   // SCL inactive state is High
    I2C_CNT_REG = (USIIFGCC);                    // Disable auto clear ctrl

   // Clear all flags
    CI_State_Machine = USI_STATE_IDLE;

    I2C_CTL0_REG &= ~(USISWRST);                 // Release USI for operation
    I2C_CTL1_REG &= ~(USIIFG);                   // Clear USI Flag

#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
    Timer_Timeout_Init();                   // Program Timer A1	CCR register
#endif

    // Initialize all callbacks
    CI_Callback_ptr = CI_Callback;
}

/******************************************************************************
 *
 * @brief   Disables the USI module
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_disable(void)
{
    I2C_CTL0_REG |= USISWRST;
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
    Timer_Timeout_Stop();
#endif
}

/******************************************************************************
 *
 * @brief   Enables the USI module
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_reenable(void)
{
    I2C_CTL0_REG &= ~USISWRST;
}

/******************************************************************************
 *
 * @brief   Checks the Stop flag and calls the callback when set
 *
 * @return  none
 *****************************************************************************/
#ifdef CONFIG_CI_PHYDL_STOP_CALLBACK
static void Check_StopFlag(void)
{
    if (I2C_CTL1_REG & USISTP)
    {
        // Reset I2C state machine
        I2C_CTL0_REG &= ~USIOE;              //Prep for Start condition
        CI_State_Machine = USI_STATE_IDLE; // Reset state machine
        I2C_CTL1_REG &= ~USISTP;
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Stop();
#endif
        if (CI_Callback_ptr->StopCallback != NULL)
            CI_Callback_ptr->StopCallback();            // Call Start callback (if enabled and valid)
    }
}
#endif

/******************************************************************************
*
 * @brief   Polls for USI flags, and maintains the USI state machine
 *  - If USISTTIFG is detected, the state machine is initialized
 *  - If USIIFG is detected, the state machine is executed
 *  - The corresponding callbacks (start, stop, TX, RX, timeout) will be called
 *    depending on the state
 *  - The callback is handled by higher lever layers
 *
 * @return  none
 *****************************************************************************/
void TI_MSPBoot_CI_PHYDL_Poll(void)
{
    uint8_t temp;

    if (I2C_CTL1_REG & USISTTIFG)                // Check for Start bit flag
    {
#ifdef CONFIG_CI_PHYDL_STOP_CALLBACK
        if (CI_State_Machine != USI_STATE_IDLE)
        {
            Check_StopFlag();
        }
#endif           
        //USI_State_Machine = USI_STATE_START_RECEIVED;
        I2C_CNT_REG &= 0xE0;                     // Clean USI bit count
        I2C_CNT_REG += 0x08;                     // Bit counter = 8
        I2C_CTL1_REG &= ~USISTTIFG;              // Clear start flag
#ifdef CONFIG_CI_PHYDL_START_CALLBACK
        if (CI_Callback_ptr->StartCallback != NULL)
            CI_Callback_ptr->StartCallback();            // Call Start callback (if enabled and valid)
#endif
        CI_State_Machine = USI_STATE_TX_ACK_ADDR;
        I2C_CTL1_REG &= ~(USIIFG|USISTP);    // Clear Start and Stop bits
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Start();
#endif
    }

    if (I2C_CTL1_REG & USIIFG)              // Check for USI flag
    {
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
        Timer_Timeout_Start();
#endif
        switch (CI_State_Machine)          // Execute state machine
        {
            case  USI_STATE_TX_ACK_ADDR:    // Address +R/W was received, send (N)ACK
                // Check the Address
                if ((I2C_SHIFT_REG_L & 0xFE) == (CONFIG_CI_PHYDL_I2C_SLAVE_ADDR << 1))
                {
                    // Message is for us, Check r/w
                    if (I2C_SHIFT_REG_L & BIT0)
                        CI_State_Machine = USI_STATE_TXBYTE;   // R/W = 1,  TX
                    else
                        CI_State_Machine = USI_STATE_RXBYTE;   // R/W = 0, RX
                    I2C_CTL0_REG |= USIOE;       // SDA = output
                    I2C_SHIFT_REG_L = 0x00;          // Send ACK
                }
                else
                {
                    // Message is not for us
                    I2C_SHIFT_REG_L = 0xFF;          // Send NACK
                    CI_State_Machine = USI_STATE_PREP_RESTART;
                }
                I2C_CNT_REG |= 0x01;             // Bit counter = 1
                I2C_CTL1_REG &= ~ USIIFG;
            break;
            case  USI_STATE_RXBYTE:
                // Receive one byte
                I2C_CTL0_REG &= ~USIOE;          // SDA = input
                I2C_CNT_REG |= 0x08;             // Bit counter = 8
                CI_State_Machine = USI_STATE_TX_ACK_DATA;
                I2C_CTL1_REG &= ~ USIIFG;
            break;
            case  USI_STATE_TX_ACK_DATA:
                // Send ACK after byte reception
                if (CI_Callback_ptr->RxCallback != NULL)
                    CI_Callback_ptr->RxCallback(I2C_SHIFT_REG_L);  // On byte RX call RXCallback if valid
                I2C_CTL0_REG |= USIOE;               // SDA output
                I2C_SHIFT_REG_L = 0;                     // Send ack
                I2C_CNT_REG |= 0x01;                 // Bit counter = 1
                CI_State_Machine = USI_STATE_RXBYTE;
                I2C_CTL1_REG &= ~ USIIFG;
            break;
            case  USI_STATE_TXBYTE:
                // Transmit one byte
                I2C_CTL0_REG |= USIOE;               // SDA = output
                if (CI_Callback_ptr->TxCallback != NULL)
                    CI_Callback_ptr->TxCallback(&temp);    // On byte TX call TXCallback if valid
                I2C_SHIFT_REG_L = temp;                  // Send Data
                I2C_CNT_REG |= 0x08;                 // Bit count = 8
                CI_State_Machine = USI_STATE_RX_ACK_DATA;
                I2C_CTL1_REG &= ~ USIIFG;
            break;
            case  USI_STATE_RX_ACK_DATA:
                // Get (N)ACK from Master after TX byte
                I2C_CTL0_REG &= ~USIOE;              // SDA = input
                I2C_CNT_REG |= 0x01;                 // Bit count = 1
                CI_State_Machine = USI_STATE_TEST_ACK;
                I2C_CTL1_REG &= ~ USIIFG;
            break;
            case  USI_STATE_TEST_ACK:
                // Test the (N)ACK from Master
                if ((I2C_SHIFT_REG_L & 0x01) == 0)
                    CI_State_Machine = USI_STATE_TXBYTE;  //Ack rxed ready for next transfer
                else
                {
                    I2C_CTL0_REG &=  ~USIOE;         // Nack rxed, last transmit
                    CI_State_Machine = USI_STATE_IDLE;   //Re-initialize state machine
                    I2C_CTL1_REG &= ~ USIIFG;
                }
            break;
            case  USI_STATE_PREP_RESTART:
                // Reset I2C state machine on address NACK
                I2C_CTL0_REG &= ~USIOE;              //Prep for Start condition
                CI_State_Machine = USI_STATE_IDLE; // Reset state machine
                I2C_CTL1_REG &= ~ USIIFG;
            break;
            default:
#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
                Timer_Timeout_Stop();
#endif
                I2C_CTL1_REG &= ~ USIIFG;           // Idle, should not get here
            break;

        }
    }
    
#ifdef CONFIG_CI_PHYDL_STOP_CALLBACK
    Check_StopFlag();   // Check the Stop flag
#endif

#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
    if (Timer_Timeout_Expired() != 0x00)
    {
        Timer_Timeout_Stop();
#ifdef CONFIG_CI_PHYDL_ERROR_CALLBACK
        if (CI_Callback_ptr->ErrorCallback != NULL)
            CI_Callback_ptr->ErrorCallback(1);          // Call Start callback (if enabled and valid)
#endif
    }
#endif
}

#ifdef CONFIG_CI_PHYDL_I2C_TIMEOUT
/******************************************************************************
*
 * @brief   Init timeout timer
 *  - Timer will run using SMCLK/4 in UP mode. Counts 25ms and is disabled by 
 *    default
 *
 * @return  none
 *****************************************************************************/
static void Timer_Timeout_Init(void)
{
    // TA Uses SMCLK/4 in Up mode and is stopped by default
    TIMER_CTL_REG = TIMER_SRC_SEL + TIMER_CLR + MC_0 + ID_2;;
    // Timer will count 25ms
    TIMER_CCR_REG = TIMEOUT_CYCLES;
    // TA.0 used in compare mode without interrupts
    TIMER_CCTL_REG = 0;;
}

/******************************************************************************
*
 * @brief   Starts timeout timer
 *
 * @return  none
 *****************************************************************************/
static void Timer_Timeout_Start(void)
{
    TIMER_CTL_REG = TIMER_SRC_SEL + TIMER_CLR + MC_1 + ID_2;    // Start timer in Up mode
    TIMER_CCTL_REG &= ~CCIFG;                         // Clear flag
}

/******************************************************************************
*
 * @brief   Stop timeout timer
 *
 * @return  none
 *****************************************************************************/
static void Timer_Timeout_Stop(void)
{
    TIMER_CTL_REG = TIMER_SRC_SEL + TIMER_CLR + MC_0;   // Stop timer
    TIMER_CCTL_REG &= ~CCIFG;                 // Clear flag
}

/******************************************************************************
*
 * @brief   Checks if timeout timer has expired
 *
 * @return  0 if not expired. !=0 if expired.
 *****************************************************************************/
static uint8_t Timer_Timeout_Expired(void)
{
    return (TIMER_CCTL_REG & CCIFG);          // Check timer flag
}

#endif  //CONFIG_CI_PHYDL_I2C_TIMEOUT

//
//  Constant table
//
/*! Peripheral Interface vectors for Application:
    These vectors are shared with application and can be used by the App to
    initialize and use the Peripheral interface
 */
#ifdef CONFIG_CI_PHYDL_COMM_SHARED
#pragma DATA_SECTION(Boot2App_Vector_Table, ".BOOT_APP_VECTORS")
#pragma RETAIN(Boot2App_Vector_Table)
const uint16_t Boot2App_Vector_Table[] =
{
    (uint16_t) &TI_MSPBoot_CI_PHYDL_Init,   /*! Initialization routine */
    (uint16_t) &TI_MSPBoot_CI_PHYDL_Poll    /*! Poll routine */
};
#endif
